"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const components_1 = require("typedoc/dist/lib/output/components");
const reflections_1 = require("typedoc/dist/lib/models/reflections");
const NavigationItem_1 = require("typedoc/dist/lib/output/models/NavigationItem");
const events_1 = require("typedoc/dist/lib/output/events");
let SdkClientTocPlugin = class SdkClientTocPlugin extends components_1.RendererComponent {
    constructor() {
        super(...arguments);
        this.commandToNavigationItems = new Map();
    }
    initialize() {
        // disable existing toc plugin
        const tocPlugin = this.owner.application.renderer.getComponent("toc");
        this.owner.off(events_1.PageEvent.BEGIN, tocPlugin.onRendererBeginPage);
        this.listenTo(this.owner, {
            [events_1.PageEvent.BEGIN]: this.onRendererBeginPage
        });
    }
    /**
     * Generates a table of contents for a page.
     * @param page Contains project details and contextual data about the page being rendered.
     */
    onRendererBeginPage(page) {
        let model = page.model;
        if (!model.constructor.name.endsWith("Reflection")) {
            return;
        }
        const trail = [];
        while (model.constructor.name !== "ProjectReflection" &&
            !model.kindOf(reflections_1.ReflectionKind.SomeModule)) {
            trail.unshift(model);
            model = model.parent;
        }
        const tocRestriction = this.owner.toc;
        page.toc = new NavigationItem_1.NavigationItem(model.name);
        if (!model.parent && !trail.length) {
            this.commandsNavigationItem = new NavigationItem_1.NavigationItem("Commands", void 0, page.toc);
            this.exceptionsNavigationItem = new NavigationItem_1.NavigationItem("Exceptions", void 0, page.toc);
        }
        this.buildToc(model, trail, page.toc, tocRestriction);
    }
    isCommand({ implementedTypes = [] }) {
        return (implementedTypes.length === 1 &&
            implementedTypes[0].type === "reference" &&
            implementedTypes[0].name === "Command");
    }
    isException(model) {
        const extendedTypes = model.extendedTypes || [];
        return (extendedTypes.length === 1 &&
            extendedTypes[0].type === "reference" &&
            extendedTypes[0].name === "ServiceException");
    }
    isUnion(model) {
        return model.type && model.type.type === "union";
    }
    isInputOrOutput(model) {
        return (model.kindString === "Interface" &&
            (model.name.endsWith("Input") || model.name.endsWith("Output")));
    }
    /**
     * Create a toc navigation item structure.
     *
     * @param model   The models whose children should be written to the toc.
     * @param trail   Defines the active trail of expanded toc entries.
     * @param parent  The parent [[NavigationItem]] the toc should be appended to.
     * @param restriction  The restricted table of contents.
     */
    buildToc(model, trail, parent, restriction) {
        const index = trail.indexOf(model);
        const children = model["children"] || [];
        if (index < trail.length - 1 && children.length > 40) {
            const child = trail[index + 1];
            const item = NavigationItem_1.NavigationItem.create(child, parent, true);
            item.isInPath = true;
            item.isCurrent = false;
            this.buildToc(child, trail, item);
        }
        else {
            children.forEach((child) => {
                if (restriction &&
                    restriction.length > 0 &&
                    restriction.indexOf(child.name) === -1) {
                    return;
                }
                if (child.kindOf(reflections_1.ReflectionKind.SomeModule)) {
                    return;
                }
                if (trail.length) {
                    const item = NavigationItem_1.NavigationItem.create(child, parent, true);
                    if (trail.indexOf(child) !== -1) {
                        item.isInPath = true;
                        item.isCurrent = trail[trail.length - 1] === child;
                        this.buildToc(child, trail, item);
                    }
                    return;
                }
                if (this.isCommand(child)) {
                    const item = NavigationItem_1.NavigationItem.create(child, this.commandsNavigationItem, true);
                    // create an entry for the command
                    const commandName = child.name.toLowerCase();
                    if (!this.commandToNavigationItems.get(commandName)) {
                        this.commandToNavigationItems.set(commandName, item);
                    }
                }
                else if (this.isException(child)) {
                    const item = NavigationItem_1.NavigationItem.create(child, this.exceptionsNavigationItem, true);
                }
                else if (this.isUnion(child) &&
                    child.type.types.every((type) => {
                        return (type.reflection &&
                            this.isException(type.reflection));
                    })) {
                    // get command from name
                    const commandName = child.name.replace("ExceptionsUnion", "").toLowerCase() + "command";
                    const item = NavigationItem_1.NavigationItem.create(child, this.commandToNavigationItems.get(commandName), true);
                }
                else if (this.isInputOrOutput(child)) {
                    // get command from name
                    const commandName = child.name.replace(/Input|Output/, "").toLowerCase() + "command";
                    const item = NavigationItem_1.NavigationItem.create(child, this.commandToNavigationItems.get(commandName), true);
                }
                else if (child.name.startsWith("_")) {
                    return;
                }
                else {
                    const item = NavigationItem_1.NavigationItem.create(child, parent, true);
                    if (trail.indexOf(child) !== -1) {
                        item.isInPath = true;
                        item.isCurrent = trail[trail.length - 1] === child;
                        this.buildToc(child, trail, item);
                    }
                }
            });
        }
    }
};
SdkClientTocPlugin = tslib_1.__decorate([
    components_1.Component({ name: "SdkClientTocPlugin" })
], SdkClientTocPlugin);
exports.SdkClientTocPlugin = SdkClientTocPlugin;
//# sourceMappingURL=sdk-client-toc-plugin.js.map